@@ -4,11 +4,13 @@ import os
 import sys
 import json
 import math
+import uuid
 import time
 
 import warnings
 import traceback
 
+import numpy as np
 import pandas as pd
 
 from ast import literal_eval
@@ -39,21 +41,23 @@ from relevanceai.constants import (
 
 
 class BatchInsertClient(BatchRetrieveClient):
-    def __init__(self, *args, **kw):
-        super().__init__(*args, **kw)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
 
     def _insert_documents(
         self,
         dataset_id: str,
         documents: list,
         bulk_fn: Callable = None,
-        max_workers: int = 8,
+        max_workers: Optional[int] = 2,
         retry_chunk_mult: float = 0.5,
         show_progress_bar: bool = False,
         chunksize: int = 0,
         use_json_encoder: bool = True,
         verbose: bool = True,
         create_id: bool = False,
+        overwrite: bool = True,
+        ingest_in_background: bool = False,
         *args,
         **kwargs,
     ):
@@ -104,17 +108,17 @@ class BatchInsertClient(BatchRetrieveClient):
         # Check if the collection exists
         self.datasets.create(dataset_id)
 
-        if use_json_encoder:
-            documents = self.json_encoder(documents)
-
-        # TODO: rename this function to convert_id_to_string
         self._convert_id_to_string(documents, create_id=create_id)
 
         def bulk_insert_func(documents):
+            if use_json_encoder:
+                documents = self.json_encoder(documents)
             return self.datasets.bulk_insert(
                 dataset_id,
                 documents,
                 return_documents=True,
+                overwrite=overwrite,
+                ingest_in_background=ingest_in_background,
                 *args,
                 **kwargs,
             )
@@ -134,148 +138,18 @@ class BatchInsertClient(BatchRetrieveClient):
             chunksize=chunksize,
         )
 
-    def _insert_csv(
-        self,
-        dataset_id: str,
-        filepath_or_buffer,
-        chunksize: int = 10000,
-        max_workers: int = 8,
-        retry_chunk_mult: float = 0.5,
-        show_progress_bar: bool = False,
-        index_col: int = None,
-        csv_args: Optional[dict] = None,
-        col_for_id: str = None,
-        auto_generate_id: bool = True,
-    ):
-
-        """
-        Insert data from csv file
-
-        Parameters
-        ----------
-        dataset_id : string
-            Unique name of dataset
-        filepath_or_buffer :
-            Any valid string path is acceptable. The string could be a URL. Valid URL schemes include http, ftp, s3, gs, and file.
-        chunksize : int
-            Number of lines to read from csv per iteration
-        max_workers : int
-            Number of workers active for multi-threading
-        retry_chunk_mult: int
-            Multiplier to apply to chunksize if upload fails
-        csv_args : dict
-            Optional arguments to use when reading in csv. For more info, see https://pandas.pydata.org/documents/reference/api/pandas.read_csv.html
-        index_col : None
-            Optional argument to specify if there is an index column to be skipped (e.g. index_col = 0)
-        col_for_id : str
-            Optional argument to use when a specific field is supposed to be used as the unique identifier ('_id')
-        auto_generate_id: bool = True
-            Automatically generateds UUID if auto_generate_id is True and if the '_id' field does not exist
-
-        Example
-        ---------
-        >>> from relevanceai import Client
-        >>> client = Client()
-        >>> df = client.Dataset("sample_dataset_id")
-        >>> df.insert_csv("temp.csv")
-
-        """
-        csv_args = {} if csv_args is None else csv_args
-
-        csv_args.pop("index_col", None)
-        csv_args.pop("chunksize", None)
-        df = pd.read_csv(
-            filepath_or_buffer, index_col=index_col, chunksize=chunksize, **csv_args
-        )
-
-        # Initialise output
-        inserted = 0
-        failed_documents = []
-        failed_documents_detailed = []
-
-        # Chunk inserts
-        for chunk in df:
-            response = self._insert_csv_chunk(
-                chunk=chunk,
-                dataset_id=dataset_id,
-                max_workers=max_workers,
-                retry_chunk_mult=retry_chunk_mult,
-                show_progress_bar=show_progress_bar,
-                col_for_id=col_for_id,
-                auto_generate_id=auto_generate_id,
-            )
-            inserted += response["inserted"]
-            failed_documents += response["failed_documents"]
-            failed_documents_detailed += response["failed_documents_detailed"]
-
-        return {
-            "inserted": inserted,
-            "failed_documents": failed_documents,
-            "failed_documents_detailed": failed_documents_detailed,
-        }
-
-    def _insert_csv_chunk(
-        self,
-        chunk,
-        dataset_id,
-        max_workers,
-        retry_chunk_mult,
-        show_progress_bar,
-        col_for_id,
-        auto_generate_id,
-    ):
-        # generate '_id' if possible
-        # col_for_id
-        if "_id" not in chunk.columns and col_for_id:
-            if col_for_id in chunk:
-                chunk.insert(0, "_id", chunk[col_for_id], False)
-            else:
-                warnings.warn(Warning.COLUMN_DNE.format(col_for_id))
-        # auto_generate_id
-        if "_id" not in chunk.columns and auto_generate_id:
-            index = chunk.index
-            uuids = [
-                make_id(chunk.iloc[chunk_index]) for chunk_index in range(len(index))
-            ]
-            chunk.insert(0, "_id", uuids, False)
-            warnings.warn(Warning.AUTO_GENERATE_IDS)
-
-        # Check for _id
-        if "_id" not in chunk.columns:
-            raise FieldNotFoundError("Need _id as a column")
-
-        # add fix for when lists are read in as strings
-        EXCEPTION_COLUMNS = ("_vector_", "_chunk_")
-        vector_columns = [i for i in chunk.columns if i.endswith(EXCEPTION_COLUMNS)]
-        for i in vector_columns:
-            chunk[i] = chunk[i].apply(literal_eval)
-
-        chunk_json = chunk.to_dict(orient="records")
-
-        print(
-            f"while inserting, you can visit your dashboard at https://cloud.relevance.ai/dataset/{dataset_id}/dashboard/monitor/"
-        )
-        response = self._insert_documents(
-            dataset_id=dataset_id,
-            documents=chunk_json,
-            max_workers=max_workers,
-            retry_chunk_mult=retry_chunk_mult,
-            show_progress_bar=show_progress_bar,
-            verbose=False,
-        )
-        return response
-
     def _update_documents(
         self,
         dataset_id: str,
         documents: list,
         bulk_fn: Callable = None,
-        max_workers: int = 8,
+        max_workers: int = 2,
         retry_chunk_mult: float = 0.5,
         chunksize: int = 0,
         show_progress_bar=False,
         use_json_encoder: bool = True,
         create_id: bool = False,
+        ingest_in_background: bool = False,
         *args,
         **kwargs,
     ):
@@ -326,14 +200,14 @@ class BatchInsertClient(BatchRetrieveClient):
         # Turn _id into string
         self._convert_id_to_string(documents, create_id=create_id)
 
-        if use_json_encoder:
-            documents = self.json_encoder(documents)
-
         def bulk_update_func(documents):
+            if use_json_encoder:
+                documents = self.json_encoder(documents)
             return self.datasets.documents.bulk_update(
                 dataset_id,
                 documents,
                 return_documents=True,
+                ingest_in_background=ingest_in_background,
                 *args,
                 **kwargs,
             )
@@ -359,7 +233,7 @@ class BatchInsertClient(BatchRetrieveClient):
         updated_documents_file: str = None,
         updating_args: Optional[dict] = None,
         retrieve_chunk_size: int = 100,
-        max_workers: int = 8,
+        max_workers: int = 2,
         filters: Optional[list] = None,
         select_fields: Optional[list] = None,
         show_progress_bar: bool = True,
@@ -547,7 +421,7 @@ class BatchInsertClient(BatchRetrieveClient):
         retrieve_chunk_size: int = 100,
         retrieve_chunk_size_failure_retry_multiplier: float = 0.5,
         number_of_retrieve_retries: int = 3,
-        max_workers: int = 8,
+        max_workers: int = 2,
         max_error: int = 1000,
         filters: Optional[list] = None,
         select_fields: Optional[list] = None,
@@ -559,7 +433,7 @@ class BatchInsertClient(BatchRetrieveClient):
         These documents are then uploaded into either an updated collection, or back into the original collection.
 
         Parameters
-        ----------
+        ------------
         original_dataset_id: string
             The dataset_id of the collection where your original documents are
         logging_dataset_id: string
@@ -731,7 +605,7 @@ class BatchInsertClient(BatchRetrieveClient):
 
     @track
     def insert_df(self, dataset_id, dataframe, *args, **kwargs):
-        """Insert a dataframe for eachd doc"""
+        """Insert a dataframe for each doc"""
 
         def _is_valid(v):
             try:
@@ -750,6 +624,144 @@ class BatchInsertClient(BatchRetrieveClient):
         self.print_search_dashboard_url(dataset_id)
         return results
 
+    def _insert_csv(
+        self,
+        dataset_id: str,
+        filepath_or_buffer,
+        id_col: str = None,
+        create_id: bool = True,
+        max_workers: int = 2,
+        retry_chunk_mult: float = 0.5,
+        show_progress_bar: bool = False,
+        csv_args: Optional[Dict[str, Any]] = None,
+        **kwargs,
+    ):
+
+        """
+        Insert data from csv file
+
+        Parameters
+        ----------
+        dataset_id : string
+            Unique name of dataset
+        filepath_or_buffer :
+            Any valid string path is acceptable. The string could be a URL. Valid URL schemes include http, ftp, s3, gs, and file.
+        id_col : str
+            Optional argument to use when a specific field is supposed to be used as the unique identifier ('_id')
+        create_id: bool = True
+            Automatically generateds UUID if create_id is True and if the '_id' field does not exist
+        max_workers : int
+            Number of workers active for multi-threading
+        retry_chunk_mult: int
+            Multiplier to apply to chunksize if upload fails
+
+        Example
+        ---------
+        >>> from relevanceai import Client
+        >>> client = Client()
+        >>> df = client.Dataset("sample_dataset_id")
+        >>> df.insert_csv("temp.csv")
+
+        """
+        df = pd.read_csv(filepath_or_buffer, **csv_args)
+
+        # Initialise output
+        inserted = 0
+        failed_documents = []
+        failed_documents_detailed = []
+
+        test_doc = json.dumps(self.json_encoder(df.loc[0].to_dict()))
+        doc_mb = sys.getsizeof(test_doc) * LIST_SIZE_MULTIPLIER / MB_TO_BYTE
+
+        target_chunk_mb = int(self.config.get_option("upload.target_chunk_mb"))
+        max_chunk_size = int(self.config.get_option("upload.max_chunk_size"))
+        chunksize = (
+            int(target_chunk_mb / doc_mb) + 1
+            if int(target_chunk_mb / doc_mb) + 1 < df.shape[0]
+            else df.shape[0]
+        )
+        chunksize = min(chunksize, max_chunk_size)
+
+        # Add edge case handling
+        if chunksize == 0:
+            chunksize = 1
+
+        # Chunk inserts
+        for chunk in self.chunk(
+            documents=df,
+            chunksize=chunksize,
+            show_progress_bar=show_progress_bar,
+        ):
+            response = self._insert_csv_chunk(
+                chunk=chunk,
+                dataset_id=dataset_id,
+                id_col=id_col,
+                create_id=create_id,
+                max_workers=max_workers,
+                retry_chunk_mult=retry_chunk_mult,
+                show_progress_bar=False,
+            )
+            inserted += response["inserted"]
+            failed_documents += response["failed_documents"]
+            failed_documents_detailed += response["failed_documents_detailed"]
+
+        return {
+            "inserted": inserted,
+            "failed_documents": failed_documents,
+            "failed_documents_detailed": failed_documents_detailed,
+        }
+
+    def _insert_csv_chunk(
+        self,
+        chunk: pd.DataFrame,
+        dataset_id: str,
+        id_col: Optional[str] = None,
+        create_id: bool = False,
+        max_workers: int = 2,
+        retry_chunk_mult: float = 0.5,
+        show_progress_bar: bool = False,
+    ):
+        # generate '_id' if possible
+        # id_col
+        if "_id" not in chunk.columns and id_col:
+            if id_col in chunk:
+                chunk.insert(0, "_id", chunk[id_col], False)
+            else:
+                warnings.warn(Warning.COLUMN_DNE.format(id_col))
+        # create_id
+        if "_id" not in chunk.columns and create_id:
+            index = chunk.index
+            uuids = [
+                make_id(chunk.iloc[chunk_index]) for chunk_index in range(len(index))
+            ]
+            chunk.insert(0, "_id", uuids, False)
+            warnings.warn(Warning.AUTO_GENERATE_IDS)
+
+        # Check for _id
+        if "_id" not in chunk.columns:
+            raise FieldNotFoundError("Need _id as a column")
+
+        # add fix for when lists are read in as strings
+        EXCEPTION_COLUMNS = ("_vector_", "_chunk_")
+        vector_columns = [i for i in chunk.columns if i.endswith(EXCEPTION_COLUMNS)]
+        for i in vector_columns:
+            chunk[i] = chunk[i].apply(literal_eval)
+
+        chunk_json = chunk.to_dict(orient="records")
+
+        print(
+            f"while inserting, you can visit your dashboard at https://cloud.relevance.ai/dataset/{dataset_id}/dashboard/monitor/"
+        )
+        response = self._insert_documents(
+            dataset_id=dataset_id,
+            documents=chunk_json,
+            max_workers=max_workers,
+            retry_chunk_mult=retry_chunk_mult,
+            show_progress_bar=show_progress_bar,
+            verbose=False,
+        )
+        return response
+
     def print_search_dashboard_url(self, dataset_id):
         search_url = (
             f"https://cloud.relevance.ai/dataset/{dataset_id}/deploy/recent/search"
@@ -779,7 +791,7 @@ class BatchInsertClient(BatchRetrieveClient):
         insert_function,
         documents: list,
         bulk_fn: Callable = None,
-        max_workers: int = 8,
+        max_workers: Optional[int] = 2,
         retry_chunk_mult: float = 0.5,
         show_progress_bar: bool = False,
         chunksize: int = 0,
@@ -795,7 +807,7 @@ class BatchInsertClient(BatchRetrieveClient):
             }
 
         # Insert documents
-        test_doc = json.dumps(documents[0], indent=4)
+        test_doc = json.dumps(self.json_encoder(documents[0]), indent=4)
         doc_mb = sys.getsizeof(test_doc) * LIST_SIZE_MULTIPLIER / MB_TO_BYTE
         if chunksize == 0:
             target_chunk_mb = int(self.config.get_option("upload.target_chunk_mb"))
@@ -806,6 +818,8 @@ class BatchInsertClient(BatchRetrieveClient):
                 else len(documents)
             )
             chunksize = min(chunksize, max_chunk_size)
+
+            print(f"Updating chunksize to {chunksize}")
             # Add edge case handling
             if chunksize == 0:
                 chunksize = 1
@@ -814,7 +828,7 @@ class BatchInsertClient(BatchRetrieveClient):
         inserted: List[str] = []
 
         # Initialise failed documents
-        failed_ids = [i["_id"] for i in documents]
+        failed_ids: List[str] = []
 
         # Initialise failed documents detailed
         failed_ids_detailed: List[str] = []
@@ -823,7 +837,7 @@ class BatchInsertClient(BatchRetrieveClient):
         cancelled_ids = []
 
         for i in range(int(self.config.get_option("retries.number_of_retries"))):
-            if len(failed_ids) > 0:
+            if len(documents) > 0:
                 self.logger.info(f"Inserting with chunksize {chunksize}")
                 if bulk_fn is not None:
                     insert_json = multiprocess(
@@ -982,6 +996,7 @@ class BatchInsertClient(BatchRetrieveClient):
                 print(
                     "❗Few errors with inserting/editing documents. Please check logs."
                 )
+                return results
 
         elif "failed_document_ids" in results:
             if len(results["failed_document_ids"]) == 0:
@@ -990,6 +1005,7 @@ class BatchInsertClient(BatchRetrieveClient):
                 print(
                     "❗Few errors with inserting/editing documents. Please check logs."
                 )
+                return results
 
         # Make backwards compatible on errors
         if (
