@@ -5,7 +5,9 @@ import collections
 import re
 import math
 import warnings
+import os
 import pandas as pd
+from relevanceai.utils.decorators.thread import fire_and_forget
 
 from tqdm.auto import tqdm
 from typing import Optional, Union, Dict, List, Mapping
@@ -201,7 +203,7 @@ class Read(Statistics):
 
     @track
     def head(
-        self, n: int = 5, raw_json: bool = False, **kw
+        self, n: int = 5, raw_json: bool = False, select_fields: list = None, **kw
     ) -> Union[dict, pd.DataFrame]:
         """
         Return the first `n` rows.
@@ -239,7 +241,7 @@ class Read(Statistics):
             f"https://cloud.relevance.ai/dataset/{self.dataset_id}/dashboard/data?page=1"
         )
         head_documents = self.get_documents(
-            number_of_documents=n, include_after_id=False
+            number_of_documents=n, include_after_id=False, select_fields=select_fields
         )
         if raw_json:
             return head_documents
@@ -475,7 +477,7 @@ class Read(Statistics):
     @track
     def schema(self) -> Dict:
         """
-        Returns the schema of a dataset. Refer to datasets.create for different field types available in a VecDB schema.
+        Returns the schema of a dataset. Refer to datasets.create for different field types available in a Relevance schema.
 
         Example
         -----------------
@@ -683,7 +685,23 @@ class Read(Statistics):
             print("✅ You have successfully inserted metadata.")
         else:
             return results
-        return self.insert_metadata(metadata)
+
+    def _update_workflow_progress(self, metadata: dict, workflow_id: str = None):
+        """
+        Updates the workflow progress
+        """
+        # RUn it asynchronously
+        if workflow_id is None:
+            workflow_id = os.getenv("WORKFLOW_ID", None)
+
+        @fire_and_forget
+        def update():
+            return self.workflows.metadata(
+                workflow_id=workflow_id, metadata=self.json_encoder(metadata)
+            )
+
+        if workflow_id is not None and workflow_id != "":
+            update()
 
     @track
     def chunk_dataset(
@@ -723,15 +741,27 @@ class Read(Statistics):
             self.dataset_id, filters=filters
         )
         with tqdm(range(math.ceil(number_of_documents / chunksize))) as pbar:
+            # update after we get the first batch
+            pbar.update(1)
+            self._update_workflow_progress(metadata=pbar.format_dict)
             while len(docs["documents"]) > 0:
+                # Update metadata if possible
+                # {'n': 0, 'total': 3, 'elapsed': 1.3828277587890625e-05,
+                # 'ncols': 143, 'nrows': 23, 'prefix': '',
+                # 'ascii': False, 'unit': 'it', 'unit_scale': False,
+                # 'rate': None, 'bar_format': None, 'postfix': None,
+                # 'unit_divisor': 1000, 'initial': 0, 'colour': None}
                 yield docs["documents"]
                 docs = self.get_documents(
                     number_of_documents=chunksize,
                     include_cursor=True,
                     after_id=docs["after_id"],
                     filters=filters,
+                    select_fields=select_fields,
                 )
+                # Final update at the end
                 pbar.update(1)
+                self._update_workflow_progress(metadata=pbar.format_dict)
         return
 
     @track
@@ -794,3 +824,20 @@ class Read(Statistics):
                     null_count[field] += 1
 
         return null_count
+
+    def facets(
+        self,
+        fields: list,
+        date_interval: str = "monthly",
+        page_size: int = 5,
+        page: int = 1,
+        asc: bool = False,
+    ):
+        return self.datasets.facets(
+            dataset_id=self.dataset_id,
+            fields=fields,
+            date_interval=date_interval,
+            page_size=page_size,
+            page=page,
+            asc=asc,
+        )
