@@ -332,7 +332,7 @@ class Operations(Write, IO):
         asc: bool
             Whether to sort results by ascending or descending order
         keep_search_history: bool
-            Whether to store the history into VecDB. This will increase the storage costs over time.
+            Whether to store the history into Relevance. This will increase the storage costs over time.
         hundred_scale: bool
             Whether to scale up the metric by 100
         search_history_id: string
@@ -409,7 +409,7 @@ class Operations(Write, IO):
         asc: bool
             Whether to sort results by ascending or descending order
         keep_search_history: bool
-            Whether to store the history into VecDB. This will increase the storage costs over time.
+            Whether to store the history into Relevance. This will increase the storage costs over time.
         hundred_scale: bool
             Whether to scale up the metric by 100
         search_history_id: string
@@ -497,7 +497,7 @@ class Operations(Write, IO):
         asc: bool
             Whether to sort results by ascending or descending order
         keep_search_history: bool
-            Whether to store the history into VecDB. This will increase the storage costs over time.
+            Whether to store the history into Relevance. This will increase the storage costs over time.
         hundred_scale: bool
             Whether to scale up the metric by 100
         query: string
@@ -585,7 +585,7 @@ class Operations(Write, IO):
         asc: bool
             Whether to sort results by ascending or descending order
         keep_search_history: bool
-            Whether to store the history into VecDB. This will increase the storage costs over time.
+            Whether to store the history into Relevance. This will increase the storage costs over time.
         hundred_scale: bool
             Whether to scale up the metric by 100
         first_step_multivector_query: list
@@ -676,11 +676,11 @@ class Operations(Write, IO):
         )
 
     @track
-    def add_sentiment(
+    def analyze_sentiment(
         self,
-        field: str,
+        text_fields: list,
+        model_name: str = "siebert/sentiment-roberta-large-english",
         output_field: str = None,
-        model_name: str = "cardiffnlp/twitter-roberta-base-sentiment",
         highlight: bool = False,
         positive_sentiment_name: str = "positive",
         max_number_of_shap_documents: Optional[int] = None,
@@ -695,7 +695,7 @@ class Operations(Write, IO):
 
         .. code-block::
 
-            ds.add_sentiment(field="sample_1_label")
+            ds.analyze_sentiment(field="sample_1_label")
 
         Parameters
         --------------
@@ -717,34 +717,18 @@ class Operations(Write, IO):
             The minimum absolute score for it to be considered important based on SHAP algorithm.
 
         """
-        from relevanceai.operations.text.sentiment import SentimentOps
+        from relevanceai.operations_new.sentiment.ops import SentimentOps
 
-        if output_field is None:
-            output_field = "_sentiment_." + field
-
-        ops = SentimentOps(model_name=model_name)
-
-        def analyze_sentiment(text):
-            return ops.analyze_sentiment(
-                text=text,
-                highlight=highlight,
-                positive_sentiment_name=positive_sentiment_name,
-                max_number_of_shap_documents=max_number_of_shap_documents,
-                min_abs_score=min_abs_score,
-            )
-
-        def analyze_sentiment_document(doc):
-            self.set_field(output_field, doc, ops.analyze_sentiment(doc.get(field, "")))
-            if doc is None:
-                return {}
-            return doc
-
-        return self.bulk_apply(
-            analyze_sentiment_document,
-            select_fields=[field],
-            **apply_args,
+        ops = SentimentOps(
+            text_fields=text_fields,
+            model_name=model_name,
+            highlight=highlight,
+            max_number_of_shap_documents=max_number_of_shap_documents,
+            min_abs_score=min_abs_score,
         )
 
+        return ops.run(self, batched=True)
+
         # return .fit_dataset(
         #     dataset=self,
         #     input_field=field,
@@ -896,12 +880,19 @@ class Operations(Write, IO):
     #     )
     #     return workflow.run(self, verbose=verbose, log_to_file=log_to_file)
 
-    def advanced_search(
+    def search(
         self,
         query: str = None,
         vector_search_query: Optional[dict] = None,
         fields_to_search: Optional[List] = None,
         select_fields: Optional[List] = None,
+        include_vectors: bool = True,
+        filters: Optional[List] = None,
+        page: int = 0,
+        page_size: int = 10,
+        sort: dict = None,
+        minimum_relevance: int = 0,
+        query_config: dict = None,
         **kwargs,
     ):
         """
@@ -925,10 +916,17 @@ class Operations(Write, IO):
             vectorSearchQuery=vector_search_query,
             fieldsToSearch=fields_to_search,
             includeFields=select_fields,
+            includeVectors=include_vectors,
+            filters=filters,
+            page=page,
+            pageSize=page_size,
+            sort=sort,
+            minimumRelevance=minimum_relevance,
+            queryConfig=query_config,
             **kwargs,
         )
 
-    search = advanced_search
+    advanced_search = search
 
     @track
     def list_deployables(self):
