@@ -1,18 +1,21 @@
 import warnings
-from relevanceai._api.api_client import (
-    APIClient,
-)  # this needs to be replace by below in dr PR
+import numpy as np
+
+from copy import deepcopy
+
+from typing import Optional, Union, Callable, Dict, Any, Set, List
 
-from relevanceai.constants.errors import MissingClusterError
 from relevanceai.utils.decorators.analytics import track
-from relevanceai.operations_new.cluster.base import ClusterBase
-from relevanceai.operations_new.apibase import OperationAPIBase
-from typing import Callable, Dict, Any, Set, List, Optional
+
+from relevanceai.operations_new.ops_base import OperationAPIBase
+from relevanceai.operations_new.cluster.transform import ClusterTransform
+
 from relevanceai.constants import Warning
-from relevanceai.dataset import Dataset
+from relevanceai.constants.errors import MissingClusterError
+from relevanceai.constants import MissingClusterError, Warning
 
 
-class ClusterOps(ClusterBase, OperationAPIBase):
+class ClusterOps(ClusterTransform, OperationAPIBase):
     """
     Cluster-related functionalities
     """
@@ -29,7 +32,7 @@ class ClusterOps(ClusterBase, OperationAPIBase):
         verbose: bool = False,
         model=None,
         model_kwargs=None,
-        *args,
+        byo_cluster_field: str = None,
         **kwargs,
     ):
         """
@@ -65,59 +68,97 @@ class ClusterOps(ClusterBase, OperationAPIBase):
         )
 
         # alias is set after model so that we can get the number of clusters
-        # if the model needs ot be instantiated
+        # if the model needs to be instantiated
         self.alias = self._get_alias(alias)
 
-    def _operate(self, cluster_id: str, field: str, output: dict, func: Callable):
+        self.byo_cluster_field = byo_cluster_field
+        if byo_cluster_field is not None:
+            self.create_byo_clusters()
+
+    def insert_centroids(
+        self,
+        centroid_documents,
+    ) -> None:
         """
-        Internal function for operations
+        Insert centroids
+        Centroids look below
 
-        It takes a cluster_id, a field, an output dictionary, and a function, and then it gets all the
-        documents in the cluster, gets the field across all the documents, and then applies the function
-        to the field
+        .. code-block::
 
-        Parameters
-        ----------
-        cluster_id : str
-            str, field: str, output: dict, func: Callable
-        field : str
-            the field you want to get the value for
-        output : dict
-            dict
-        func : Callable
-            Callable
+            cluster_ops = client.ClusterOps(
+                vector_field=["sample_1_vector_"],
+                alias="sample"
+            )
+            cluster_ops.insert_centroids(
+                centorid_documents=[
+                    {"_id" : "cluster-0", "sample_1_vector_": [1, 1, 1]},
+                    {"_id" : "cluster-1", "sample_1_vector_": [1, 2, 2]},
+                ]
+            )
 
         """
-        cluster_field = self._get_cluster_field_name()
-        # TODO; change this to fetch all documents
-        documents = self.datasets.documents.get_where(
-            self.dataset_id,
-            filters=[
-                {
-                    "field": cluster_field,
-                    "filter_type": "exact_match",
-                    "condition": "==",
-                    "condition_value": cluster_id,
-                },
-                {
-                    "field": field,
-                    "filter_type": "exists",
-                    "condition": ">=",
-                    "condition_value": " ",
-                },
-            ],
-            select_fields=[field, cluster_field],
-            page_size=9999,
+        # Centroid documents are in the format {"cluster-0": [1, 1, 1]}
+        return self.datasets.cluster.centroids.insert(
+            dataset_id=self.dataset_id,
+            cluster_centers=self.json_encoder(centroid_documents),
+            vector_fields=self.vector_fields,
+            alias=self.alias,
         )
-        # get the field across each
-        arr = self.get_field_across_documents(field, documents["documents"])
-        output[cluster_id] = func(arr)
 
-    def _operate_across_clusters(self, field: str, func: Callable):
-        output: Dict[str, Any] = dict()
-        for cluster_id in self.list_cluster_ids():
-            self._operate(cluster_id=cluster_id, field=field, output=output, func=func)
-        return output
+    def calculate_centroids(self):
+
+        # calculate the centroids
+        centroid_vectors = {}
+
+        def calculate_centroid(vectors):
+            X = np.array(vectors)
+            return X.mean(axis=0)
+
+        centroid_vectors = self._operate_across_clusters(
+            field=self.vector_fields[0], func=calculate_centroid
+        )
+
+        # Does this insert properly?
+        if isinstance(centroid_vectors, dict):
+            centroid_vectors = [
+                {"_id": k, self.vector_fields[0]: v}
+                for k, v in centroid_vectors.items()
+            ]
+        return centroid_vectors
+
+    def create_centroids(self, insert: bool = True):
+        """
+        Calculate centroids from your dataset vectors.
+
+        Example
+        --------
+
+        .. code-block::
+
+            from relevanceai import Client
+            client = Client()
+            ds = client.Dataset("sample")
+            cluster_ops = ds.ClusterOps(
+                alias="kmeans-25",
+                vector_fields=['sample_vector_']
+            )
+            centroids = cluster_ops.create_centroids()
+
+        """
+        # Get an array of the different vectors
+        if len(self.vector_fields) > 1:
+            raise NotImplementedError(
+                "Do not currently support multiple vector fields for centroid creation."
+            )
+
+        # calculate the centroids
+        centroid_vectors = self.calculate_centroids()
+
+        if insert:
+            self.insert_centroids(
+                centroid_documents=centroid_vectors,
+            )
+        return centroid_vectors
 
     def list_cluster_ids(
         self,
@@ -150,6 +191,8 @@ class ClusterOps(ClusterBase, OperationAPIBase):
             The number of clusters
 
         """
+        if alias is None:
+            alias = self.alias
         # Mainly to be used for subclustering
         # Get the cluster alias
         cluster_field = self._get_cluster_field_name()
@@ -181,97 +224,13 @@ class ClusterOps(ClusterBase, OperationAPIBase):
 
         return list(all_cluster_ids)
 
-    def insert_centroids(
-        self,
-        centroid_documents,
-    ) -> None:
-        """
-        Insert centroids
-        Centroids look below
-
-        .. code-block::
-
-            cluster_ops = client.ClusterOps(
-                vector_field=["sample_1_vector_"],
-                alias="sample"
-            )
-            cluster_ops.insert_centroids(
-                centorid_documents={
-                    "cluster-0": [1, 1, 1],
-                    "cluster-2": [2, 1, 1]
-                }
-            )
-
-        """
-        # Centroid documents are in the format {"cluster-0": [1, 1, 1]}
-        return self.datasets.cluster.centroids.insert(
-            dataset_id=self.dataset_id,
-            cluster_centers=self.json_encoder(centroid_documents),
-            vector_fields=self.vector_fields,
-            alias=self.alias,
-        )
-
-    def create_centroids(self):
-        """
-        Calculate centroids from your vectors
-
-        Example
-        --------
-
-        .. code-block::
-
-            from relevanceai import Client
-            client = Client()
-            ds = client.Dataset("sample")
-            cluster_ops = ds.ClusterOps(
-                alias="kmeans-25",
-                vector_fields=['sample_vector_']
-            )
-            centroids = cluster_ops.create_centroids()
-
-        """
-        # Get an array of the different vectors
-        if len(self.vector_fields) > 1:
-            raise NotImplementedError(
-                "Do not currently support multiple vector fields for centroid creation."
-            )
-
-        # calculate the centroids
-        centroid_vectors = self.calculate_centroids()
-
-        self.insert_centroids(
-            centroid_documents=centroid_vectors,
-        )
-        return centroid_vectors
-
-    def calculate_centroids(self):
-        import numpy as np
-
-        # calculate the centroids
-        centroid_vectors = {}
-
-        def calculate_centroid(vectors):
-            X = np.array(vectors)
-            return X.mean(axis=0)
-
-        centroid_vectors = self._operate_across_clusters(
-            field=self.vector_fields[0], func=calculate_centroid
-        )
-
-        # Does this insert properly?
-        if isinstance(centroid_vectors, dict):
-            centroid_vectors = [
-                {"_id": k, self.vector_fields[0]: v}
-                for k, v in centroid_vectors.items()
-            ]
-        return centroid_vectors
-
     def get_centroid_documents(self):
         centroid_vectors = {}
-        if self.model.cluster_centers_ is not None:
-            centroid_vectors = self.model.cluster_centers_
+        if hasattr(self.model, "_centroids") and self.model._centroids is not None:
+            centroid_vectors = self.model._centroids
             # get the cluster label function
-            cluster_ids = self.list_cluster_ids()
+            labels = range(len(centroid_vectors))
+            cluster_ids = self.format_cluster_labels(labels)
             if len(self.vector_fields) > 1:
                 warnings.warn(
                     "Currently do not support inserting centroids with multiple vector fields"
@@ -282,9 +241,56 @@ class ClusterOps(ClusterBase, OperationAPIBase):
             ]
         else:
             centroids = self.create_centroids()
-
         return centroids
 
+    @property
+    def centroids(self):
+        """
+        Access the centroids of your dataset easily
+
+        .. code-block::
+
+            ds = client.Dataset("sample")
+            cluster_ops = ds.ClusterOps(
+                vector_fields=["sample_vector_"],
+                alias="simple"
+            )
+            cluster_ops.centroids
+
+        """
+        if not hasattr(self, "_centroids"):
+            self._centroids = self.datasets.cluster.centroids.documents(
+                dataset_id=self.dataset_id,
+                vector_fields=self.vector_fields,
+                alias=self.alias,
+                page_size=9999,
+                include_vector=True,
+            )["results"]
+        return self._centroids
+
+    def get_centroid_from_id(
+        self,
+        cluster_id: str,
+    ) -> Dict[str, Any]:
+        """> It takes a cluster id and returns the centroid with that id
+
+        Parameters
+        ----------
+        cluster_id : str
+            The id of the cluster to get the centroid for.
+
+        Returns
+        -------
+            The centroid with the given id.
+
+        """
+
+        for centroid in self.centroids:
+            if centroid["_id"] == cluster_id:
+                return centroid
+
+        raise ValueError(f"Missing the centorid with id {cluster_id}")
+
     def list_closest(
         self,
         cluster_ids: Optional[list] = None,
@@ -363,7 +369,6 @@ class ClusterOps(ClusterBase, OperationAPIBase):
             verbose=verbose,
         )
 
-    @track
     def list_furthest(
         self,
         cluster_ids: Optional[List] = None,
@@ -444,7 +449,8 @@ class ClusterOps(ClusterBase, OperationAPIBase):
         encode_fn_or_model,
         n_closest: int = 5,
         highlight_output_field="_explain_",
-        algorithm: str = "centroid",
+        algorithm: str = "relational",
+        model_kwargs: Optional[dict] = None,
     ):
         """
         It takes a text field and a function that encodes the text field into a vector.
@@ -453,6 +459,7 @@ class ClusterOps(ClusterBase, OperationAPIBase):
             def encode(X):
                 return [1, 2, 1]
             cluster_ops.explain_text_clusters(text_field="hey", encode_fn_or_model=encode)
+
         Parameters
         ----------
         text_field
@@ -465,17 +472,19 @@ class ClusterOps(ClusterBase, OperationAPIBase):
             The name of the field that will be added to the output dataset.
         algorithm: str
             Algorithm is either "centroid" or "relational"
+
         Returns
         -------
             A new dataset with the same data as the original dataset, but with a new field called _explain_
         """
         if isinstance(encode_fn_or_model, str):
             # Get the model
-            self.vectorizer = self._get_model(encode_fn_or_model)
-            if hasattr(self.vectorizer, "encode"):
-                encode_fn = self.vectorizer.encode
-            else:
-                raise AttributeError("Vectorizer is missing an `encode` function.")
+            from relevanceai.operations_new.vectorize.text.transform import (
+                VectorizeTextTransform,
+            )
+
+            self.model = VectorizeTextTransform._get_model(encode_fn_or_model)
+            encode_fn = self.model.encode
         else:
             encode_fn = encode_fn_or_model
 
@@ -506,55 +515,6 @@ class ClusterOps(ClusterBase, OperationAPIBase):
             )
         raise ValueError("Algorithm needs to be either `relational` or `centroid`.")
 
-    def _get_n_clusters(self):
-        if "n_clusters" in self.model_kwargs:
-            return self.model_kwargs["n_clusters"]
-        elif hasattr(self.model, "n_clusters"):
-            return self.model.n_clusters
-        elif hasattr(self.model, "k"):
-            return self.model.k
-        return None
-
-    def _generate_alias(self) -> str:
-        # Issue a warning about auto-generated alias
-        # We auto-generate certain aliases if the model
-        # is a default model like kmeans or community detection
-        n_clusters = self._get_n_clusters()
-        if hasattr(self.model, "alias"):
-            return self.model.alias
-
-        if n_clusters is not None:
-            alias = f"{self.model_name}-{n_clusters}"
-        else:
-            alias = f"{self.model_name}"
-        if self.verbose:
-            print(f"The alias is `{alias.lower()}`.")
-
-        Warning.MISSING_ALIAS.format(alias=alias)
-        return alias
-
-    def _get_alias(self, alias: Any) -> str:
-        # Depending a package
-        # Get the alias
-        self._get_package_from_model(self.model)
-        if self.package == "sklearn":
-            self.alias = self._get_alias_from_sklearn()
-            if self.alias is not None:
-                return self.alias
-
-        if alias is not None and isinstance(alias, str):
-            return alias
-
-        alias = self._generate_alias()
-        return alias.lower()
-
-    def _get_alias_from_sklearn(self):
-        from sklearn.cluster import KMeans
-
-        if isinstance(self.model, KMeans):
-            return "kmeans-" + str(self.model.n_clusters)
-        return None
-
     def store_operation_metadatas(self):
         self.store_operation_metadata(
             operation="cluster",
@@ -563,68 +523,141 @@ class ClusterOps(ClusterBase, OperationAPIBase):
                     "model": self.model,
                     "vector_fields": self.vector_fields,
                     "alias": self.alias,
-                    "filters": self.filters,
                     "model_kwargs": self.model_kwargs,
                 }
             ),
         )
 
-    def run(
-        self,
-        dataset: Dataset,
-        select_fields: list = None,
-        filters: list = None,
-        *args,
-        **kwargs,
-    ):
-        # A default run on all datasets
-        documents = dataset.get_all_documents(
-            select_fields=select_fields, filters=filters
-        )
-        # Loop through all documents
-        updated_documents = self.transform(documents, *args, **kwargs)
-        results = dataset.upsert_documents(updated_documents)
-        # insert centroids
-        # TODO: update values
-        centroid_documents = self.get_centroid_documents()
-        self.insert_centroids(centroid_documents)
+    @staticmethod
+    def _get_filters(
+        filters: List[Dict[str, Union[str, int]]],
+        vector_fields: List[str],
+    ) -> List[Dict[str, Union[str, int]]]:
+        """It takes a list of filters and a list of vector fields and returns a list of filters that
+        includes the original filters and a filter for each vector field that checks if the vector field
+        exists
 
-        self.store_operation_metadata(
-            dataset=dataset,
-            values={"select_fields": select_fields, **kwargs},
+        Parameters
+        ----------
+        filters : List[Dict[str, Union[str, int]]]
+            List[Dict[str, Union[str, int]]]
+        vector_fields : List[str]
+            List[str] = ["vector_field_1", "vector_field_2"]
+
+        Returns
+        -------
+            A list of dictionaries.
+
+        """
+
+        vector_field_filters = [
+            {
+                "field": vector_field,
+                "filter_type": "exists",
+                "condition": ">=",
+                "condition_value": " ",
+            }
+            for vector_field in vector_fields
+        ]
+
+        filters = deepcopy(filters)
+
+        if filters is None:
+            filters = vector_field_filters
+        else:
+            filters += vector_field_filters  # type: ignore
+
+        return filters
+
+    def merge(self, target_cluster_id: str, cluster_ids: list):
+        """
+        Merge clusters into the target cluster.
+        The centroids are re-calculated and become a new middle.
+        """
+        return self.datasets.cluster.merge(
+            dataset_id=self.dataset_id,
+            vector_fields=self.vector_fields,
+            alias=self.alias,
+            cluster_ids=[target_cluster_id] + cluster_ids,
         )
+
+    def create_byo_clusters(self):
+        """
+        Create BYO clusters for a given field
+        """
+        # TODO: Change into generator to make unique values more than 9999
+        results = self.datasets.facets(
+            dataset_id=self.dataset_id, fields=[self.byo_cluster_field], page_size=9999
+        )
+
+        try:
+            for r in results["results"][self.byo_cluster_field]:
+                filters = [
+                    {
+                        "field": self.byo_cluster_field,
+                        "filter_type": "exact_match",
+                        "condition": "==",
+                        "condition_value": r["value"],
+                    }
+                ]
+                cluster_doc = {}
+                self.set_field(self._get_cluster_field_name(), cluster_doc, r["value"])
+                results = self.datasets.documents.update_where(
+                    dataset_id=self.dataset_id, update=cluster_doc, filters=filters
+                )
+
+        except KeyError:
+            raise ValueError("Cluster field has no values.")
+
         return results
 
-    @property
-    def centroids(self):
+    def _operate(self, cluster_id: str, field: str, output: dict, func: Callable):
         """
-        Access the centroids of your dataset easily
+        Internal function for operations
 
-        .. code-block::
+        It takes a cluster_id, a field, an output dictionary, and a function, and then it gets all the
+        documents in the cluster, gets the field across all the documents, and then applies the function
+        to the field
 
-            ds = client.Dataset("sample")
-            cluster_ops = ds.ClusterOps(
-                vector_fields=["sample_vector_"],
-                alias="simple"
-            )
-            cluster_ops.centroids
+        Parameters
+        ----------
+        cluster_id : str
+            str, field: str, output: dict, func: Callable
+        field : str
+            the field you want to get the value for
+        output : dict
+            dict
+        func : Callable
+            Callable
 
         """
-        if not hasattr(self, "_centroids"):
-            self._centroids = self.datasets.cluster.centroids.documents(
-                dataset_id=self.dataset_id,
-                vector_fields=self.vector_fields,
-                alias=self.alias,
-                page_size=9999,
-                include_vector=True,
-            )["results"]
-        return self._centroids
+        cluster_field = self._get_cluster_field_name()
+        # TODO; change this to fetch all documents
+        documents = self._get_all_documents(
+            self.dataset_id,
+            filters=[
+                {
+                    "field": cluster_field,
+                    "filter_type": "exact_match",
+                    "condition": "==",
+                    "condition_value": cluster_id,
+                },
+                {
+                    "field": field,
+                    "filter_type": "exists",
+                    "condition": ">=",
+                    "condition_value": " ",
+                },
+            ],
+            select_fields=[field, cluster_field],
+            show_progress_bar=False,
+        )
+        # get the field across each
+        arr = self.get_field_across_documents(field, documents)
+        output[cluster_id] = func(arr)
 
-    def get_centroid_from_id(self, cluster_id: str):
-        """
-        Get the centroid from the relevant document ID
-        """
-        for c in self.centroids:
-            if c["_id"] == cluster_id:
-                return c
-        raise ValueError(f"Missing the centorid with id {cluster_id}")
+    def _operate_across_clusters(self, field: str, func: Callable):
+        output: Dict[str, Any] = dict()
+        for cluster_id in self.list_cluster_ids():
+            self._operate(cluster_id=cluster_id, field=field, output=output, func=func)
+        return output
