@@ -20,7 +20,6 @@ from requests import Request
 
 from relevanceai.constants.config import Config
 from relevanceai.utils.logger import AbstractLogger, FileLogger
-from relevanceai.dashboard.dashboard_mappings import DASHBOARD_MAPPINGS
 from relevanceai.constants.errors import APIError
 from relevanceai.utils.json_encoder import JSONEncoderUtils
 from relevanceai.utils.config_mixin import ConfigMixin
@@ -28,6 +27,12 @@ from relevanceai.utils.config_mixin import ConfigMixin
 DO_NOT_REPEAT_STATUS_CODES = {400, 401, 413, 404, 422}
 _HAS_PRINTED = False
 
+DASHBOARD_MAPPINGS = {
+    "multivector_search": "/sdk/search",
+    "cluster_centroids_closest": "/sdk/cluster/centroids/closest",
+    "cluster_centroids_furthest": "/sdk/cluster/centroids/furthest",
+    "cluster_aggregation": "/sdk/cluster/aggregation",
+}
 
 class Transport(JSONEncoderUtils, ConfigMixin):
     """_Base class for all relevanceai objects"""
@@ -51,22 +56,21 @@ class Transport(JSONEncoderUtils, ConfigMixin):
             dir = user_cache_dir("relevanceai", version=__version__)
             os.makedirs(dir, exist_ok=True)
 
-            self.request_logging_fpath = os.path.join(
-                dir, request_log_filename
-            ).replace("\\", "/")
+            self.request_logging_fpath = os.path.join(dir, request_log_filename)
 
             from relevanceai.utils import FileLogger
 
-            self.request_logger = FileLogger(fn=self.request_logging_fpath)
-            self.hooks = {"response": self.log}
+            self.request_logger = FileLogger(
+                fn=self.request_logging_fpath, verbose=True
+            )
+            # if hasattr(self.request_logger, "log"):
+            #     self.hooks = {"response": self.request_logger.log}
+            self.hooks = None
 
         else:
             self.hooks = None
 
-        for k, v in kwargs.items():
-            setattr(self, k, v)
-
-    def log(self, response, *args, **kwargs):
+    def log_response_to_file(self, response, *args, **kwargs):
         """It takes the response from the request and logs the url, path_url, method, status_code, headers,
         content, time, and elapsed time
 
@@ -96,7 +100,6 @@ class Transport(JSONEncoderUtils, ConfigMixin):
 
             response = {"send": log, "recv": content}
             pprint(response, sort_dicts=False)
-
             print()
             print()
 
@@ -299,7 +302,8 @@ class Transport(JSONEncoderUtils, ConfigMixin):
 
                 with requests.Session() as s:
                     response = s.send(req)
-
+                if hasattr(self, "request_logger"):
+                    self.log_response_to_file(response)
                 # Successful response
                 if response.status_code == 200:
                     self._log_response_success(base_url, endpoint)
@@ -408,7 +412,7 @@ class Transport(JSONEncoderUtils, ConfigMixin):
                 ) as response:
 
                     if os.getenv("DEBUG_REQUESTS") == "TRUE":
-                        self.log(response)
+                        self.log_response_to_file(response)
 
                     if response.status == 200:
                         self._log_response_success(base_url, endpoint)
